var WordCountPlus = {

	selectedWords: 0,
	totalWords: 0,
	previousTotal: -1,
	selectedChars: 0,
	totalChars: 0,
	previousCharTotal: -1,
	undoAllowed: false,
	autoCopy: false,
	autoDeselect: false,
	preCountText: "",
  postCountText: "",
  thousandSep: "",
	prefs: null,
	options: null,
	observerService: null,
	clipboardHelper: null,

	init: function() {	
		// Get all the prefs
		WordCountPlus.prefs = Components.classes["@mozilla.org/preferences-service;1"]
													.getService(Components.interfaces.nsIPrefBranch);
		WordCountPlus.getCounterState();
		
		// Get and monitor the preferences that are set in the options dialogue
		WordCountPlus.options = Components.classes["@mozilla.org/preferences-service;1"]
         										.getService(Components.interfaces.nsIPrefService)
		         										.getBranch("extensions.wordcountplus.options.");
		WordCountPlus.options.QueryInterface(Components.interfaces.nsIPrefBranch2);
		WordCountPlus.options.addObserver("", WordCountPlus, false);
		WordCountPlus.getOptions();
		
		// Show Word Count Plus in the status bar
		WordCountPlus.updateStatus();
		
		// Watch for when the main area context menu is showing -- need to hide menu options if no text selected
		document.getElementById("contentAreaContextMenu").addEventListener("popupshowing", WordCountPlus.contextItemsHide, false);
		
		// Watch for the application quit notification
		WordCountPlus.observerService = Components.classes["@mozilla.org/observer-service;1"]
	                                  .getService(Components.interfaces.nsIObserverService);
		WordCountPlus.observerService.addObserver(WordCountPlus, "quit-application-granted", false);
		
		// Set up access to the clipboard
		WordCountPlus.clipboardHelper = Components.classes["@mozilla.org/widget/clipboardhelper;1"].getService(Components.interfaces.nsIClipboardHelper);
	},
	
	handleStatusClick: function(clickEvent) {
		if (clickEvent.button == 0 && clickEvent.shiftKey) {
			// on shift-left-click, count total words
			WordCountPlus.countTotalWords();
		}
		else if ((clickEvent.button == 1) || (clickEvent.button == 0 &&  clickEvent.altKey)) {
			// on middle click or alt-left-click, reset the counts 
			WordCountPlus.zeroCounts();
		}
		else if (clickEvent.button == 0) {
			// on left click, no key modifiers, count selected words
			WordCountPlus.countSelectedWords();
		}
	},
	
	openOptionsWindow: function() {
		// Only open the options window if it's not already open -- otherwise just focus it              
		var wm = Components.classes['@mozilla.org/appshell/window-mediator;1']
                        .getService(Components.interfaces.nsIWindowMediator);
		var optionsWin = wm.getMostRecentWindow("mozilla:wordcountplus-options");
 		if (optionsWin) {
 			optionsWin.focus();
		}
		else {
			var instantApply = WordCountPlus.prefs.getBoolPref("browser.preferences.instantApply", false);
			var features = "chrome,titlebar,resizable=no,centerscreen" + (instantApply ? ",dialog=no" : "");
			window.openDialog("chrome://wordcountplus/content/options.xul", "", features);
		}
	},
		
	showPanel: function() {
		// When a window is activated, retrieve the current counter state and 
		// make the wordcount panel visible, hiding it in inactive panes
		var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                   .getService(Components.interfaces.nsIWindowMediator);
		var enumerator = wm.getEnumerator("navigator:browser");
		while(enumerator.hasMoreElements()) {
			var win = enumerator.getNext();
			win.document.getElementById("wordcountplus-panel").collapsed = true;
		}
		WordCountPlus.getCounterState();
		WordCountPlus.updateStatus();
		document.getElementById("wordcountplus-panel").collapsed = false;		
	},
	
	doCount: function(sel) {
		var c = sel;
		if (c) {
			c = c.replace(/\s/g,' ').replace(/ {2,}/g,' ').replace(/ $|^ /g,'').split(' ');
			c = c.length;
		}
		else {
			alert(document.getElementById("wordcountplus-strings").getString("nothingSelectedMessage"))
			c = 0;
		}
		return c;
	},
	
	getSelectedText: function() {
		var c = document.commandDispatcher.focusedWindow.getSelection().toString();
		if (c) {
			return c;	
		}
		else {
			// Nothing selected in the window, try for a text box
			var e = document.commandDispatcher.focusedElement;
			if (e) {
				if ((e.tagName.toLowerCase() == "input" && (e.type == "text")) || (e.tagName.toLowerCase() == "textarea") || (e.tagName.toLowerCase() == "textbox")) {
					var st = e.selectionStart;
					var end = e.selectionEnd;
					if (end > st) {
						c = e.value.substring(st, end);
						return c;
					}
				}
			}
		}
		return "";	
	},
	
	collapseSelected: function() {
		var c = document.commandDispatcher.focusedWindow.getSelection().toString();
		if (c) {
			document.commandDispatcher.focusedWindow.getSelection().collapseToStart();
		}
		else {
			// Nothing selected in the window, try for a text box
			var e = document.commandDispatcher.focusedElement;
			if (e) {
				if ((e.tagName.toLowerCase() == "input" && (e.type == "text")) || (e.tagName.toLowerCase() == "textarea") || (e.tagName.toLowerCase() == "textbox")) {
					e.setSelectionRange(e.selectionStart, e.selectionStart);
				}
			}
		}	
	},
	
	countSelectedWords: function() {	
		var sel = WordCountPlus.getSelectedText();
		var c = WordCountPlus.doCount(sel);
		if (c != 0) {
			WordCountPlus.selectedWords = c;
			WordCountPlus.selectedChars = sel.length;
			WordCountPlus.prefs.setIntPref("extensions.wordcountplus.selectedWords", WordCountPlus.selectedWords);
			WordCountPlus.prefs.setIntPref("extensions.wordcountplus.selectedChars", WordCountPlus.selectedChars);
			WordCountPlus.updateStatus();
			
			if (WordCountPlus.autoCopy) {
				WordCountPlus.doCopy(WordCountPlus.selectedWords);
			}
			
			if (WordCountPlus.autoDeselect) {
				// deselect the selection		
				WordCountPlus.collapseSelected();	
			}
		}
	},
	
	countTotalWords: function() {
		var existingWordCount = 0;
		var existingCharCount = 0;
		if (WordCountPlus.totalWords == 0) {
			existingWordCount = WordCountPlus.selectedWords;		
			existingCharCount = WordCountPlus.selectedChars;		
		}	
		var sel = WordCountPlus.getSelectedText();
		var c = WordCountPlus.doCount(sel);
		if (c != 0) {
			WordCountPlus.selectedWords = c;
			WordCountPlus.previousTotal = WordCountPlus.totalWords;
			WordCountPlus.totalWords = WordCountPlus.totalWords + c + existingWordCount;
			WordCountPlus.selectedChars = sel.length;
			WordCountPlus.previousCharTotal = WordCountPlus.totalChars;
			WordCountPlus.totalChars= WordCountPlus.totalChars + WordCountPlus.selectedChars + existingCharCount;
			WordCountPlus.undoAllowed = true;
			WordCountPlus.prefs.setIntPref("extensions.wordcountplus.selectedWords", WordCountPlus.selectedWords);
			WordCountPlus.prefs.setIntPref("extensions.wordcountplus.totalWords", WordCountPlus.totalWords);
			WordCountPlus.prefs.setIntPref("extensions.wordcountplus.previousTotal", WordCountPlus.previousTotal);
			WordCountPlus.prefs.setIntPref("extensions.wordcountplus.selectedChars", WordCountPlus.selectedChars);
			WordCountPlus.prefs.setIntPref("extensions.wordcountplus.totalChars", WordCountPlus.totalChars);
			WordCountPlus.prefs.setIntPref("extensions.wordcountplus.previousCharTotal", WordCountPlus.previousCharTotal);
			WordCountPlus.prefs.setBoolPref("extensions.wordcountplus.undoAllowed", WordCountPlus.undoAllowed);
			WordCountPlus.updateStatus();
			
			if (WordCountPlus.autoCopy) {
				WordCountPlus.doCopy(WordCountPlus.totalWords);
			}
			
			if (WordCountPlus.autoDeselect) {
				// deselect the selection	
				WordCountPlus.collapseSelected();			
			}
		}
	},
	
	zeroCounts: function() {
		if (WordCountPlus.selectedWords != 0 || WordCountPlus.totalWords != 0) {
			if (WordCountPlus.selectedWords != 0 ) {
				WordCountPlus.selectedWords = 0;
				WordCountPlus.selectedChars = 0;
				WordCountPlus.prefs.setIntPref("extensions.wordcountplus.selectedWords", WordCountPlus.selectedWords);
				WordCountPlus.prefs.setIntPref("extensions.wordcountplus.selectedChars", WordCountPlus.selectedChars);
			}
			if (WordCountPlus.totalWords != 0) {
				WordCountPlus.previousTotal = WordCountPlus.totalWords;
				WordCountPlus.totalWords = 0;
				WordCountPlus.previousCharTotal = WordCountPlus.totalChars;
				WordCountPlus.totalChars = 0;
				WordCountPlus.undoAllowed = true;
				WordCountPlus.prefs.setIntPref("extensions.wordcountplus.totalWords", WordCountPlus.totalWords);
				WordCountPlus.prefs.setIntPref("extensions.wordcountplus.previousTotal", WordCountPlus.previousTotal);
				WordCountPlus.prefs.setIntPref("extensions.wordcountplus.totalChars", WordCountPlus.totalChars);
				WordCountPlus.prefs.setIntPref("extensions.wordcountplus.previousCharTotal", WordCountPlus.previousCharTotal);
				WordCountPlus.prefs.setBoolPref("extensions.wordcountplus.undoAllowed", WordCountPlus.undoAllowed);
			}
			WordCountPlus.updateStatus();
		}
	},
	
	undoredoCounter: function() {
		if (WordCountPlus.previousTotal >= 0) {
			var currentTotal = WordCountPlus.totalWords;
			var currentCharTotal = WordCountPlus.totalChars;
			WordCountPlus.totalWords = WordCountPlus.previousTotal;
			WordCountPlus.previousTotal = currentTotal;
			WordCountPlus.totalChars = WordCountPlus.previousCharTotal;
			WordCountPlus.previousCharTotal = currentCharTotal;
			WordCountPlus.undoAllowed = !WordCountPlus.undoAllowed;
			WordCountPlus.prefs.setIntPref("extensions.wordcountplus.totalWords", WordCountPlus.totalWords);
			WordCountPlus.prefs.setIntPref("extensions.wordcountplus.previousTotal", WordCountPlus.previousTotal);
			WordCountPlus.prefs.setIntPref("extensions.wordcountplus.totalChars", WordCountPlus.totalChars);
			WordCountPlus.prefs.setIntPref("extensions.wordcountplus.previousCharTotal", WordCountPlus.previousCharTotal);
			WordCountPlus.prefs.setBoolPref("extensions.wordcountplus.undoAllowed", WordCountPlus.undoAllowed);
			WordCountPlus.updateStatus();
		}
	},
	
	updateStatus: function() {
		var wordStatus = document.getElementById("wordcountplus-panel");
		var wordtext = "";
		if (WordCountPlus.selectedWords != 0) {
			wordtext = "words";
			switch (WordCountPlus.selectedWords) {
				case 1:
				 wordtext = document.getElementById("wordcountplus-strings").getString("wordsOne");
				 break;
				case 2:
				 wordtext = document.getElementById("wordcountplus-strings").getString("wordsTwo");
				 break;
				default:
				 wordtext = document.getElementById("wordcountplus-strings").getString("wordsMore");
			}
			wordtext = WordCountPlus.addThousandSep(WordCountPlus.selectedWords) + ' ' + wordtext;
		}
		if (WordCountPlus.totalWords != 0) {			
			var totaltext = document.getElementById("wordcountplus-strings").getString("totalWords");
			wordtext = (wordtext != "") ? wordtext + ' | ' : wordtext;
			wordtext = wordtext + WordCountPlus.addThousandSep(WordCountPlus.totalWords) + ' ' + totaltext;
		}
		
		// Need to change the style of the panel depending on whether there is text or not, to 
		// get nice even spacing in all cases.
		if (wordtext == "") {
			wordStatus.setAttribute("class", "statusbarpanel-iconic");
			wordStatus.setAttribute("src", "chrome://wordcountplus/skin/wcp-small.png");	
		}
		else {
			wordStatus.setAttribute("class", "statusbarpanel-iconic-text");
			wordStatus.setAttribute("src", "chrome://wordcountplus/skin/wcp-small.png");
		}
		wordStatus.label = wordtext;
	},
	
	updateTooltip: function() {
		var chartext = "";
		if (WordCountPlus.selectedChars != 0) {
			chartext = "chars";
			switch (WordCountPlus.selectedChars) {
				case 1:
				 chartext = document.getElementById("wordcountplus-strings").getString("charsOne");
				 break;
				case 2:
				 chartext = document.getElementById("wordcountplus-strings").getString("charsTwo");
				 break;
				default:
				 chartext = document.getElementById("wordcountplus-strings").getString("charsMore");
			}
			chartext = WordCountPlus.addThousandSep(WordCountPlus.selectedChars) + ' ' + chartext;
		}
		if (WordCountPlus.totalChars != 0) {			
			var totaltext = document.getElementById("wordcountplus-strings").getString("totalChars");
			chartext = (chartext != "") ? chartext + ' | ' : chartext;
			chartext = chartext + WordCountPlus.addThousandSep(WordCountPlus.totalChars) + ' ' + totaltext;
		}
		if (chartext == "") {
			chartext = document.getElementById("wordcountplus-strings").getString("wcpName");			
		}
		document.getElementById("wordcountplus-tooltiptext").setAttribute("value", chartext);
	},
	
	updateStatusContext: function() {		
		// Count Actions
		var sel = WordCountPlus.getSelectedText();
		var noSelection = (sel == "") ? true : false;
  	document.getElementById("wordcountplus-statuscount").setAttribute("disabled", noSelection);
	  document.getElementById("wordcountplus-statusadd").setAttribute("disabled", noSelection);
		
		// Undo-Redo
		var undoMenuItem = document.getElementById("wordcountplus-undo");
		var redoMenuItem = document.getElementById("wordcountplus-redo");
		if (WordCountPlus.previousTotal < 0) {
			undoMenuItem.setAttribute("disabled", "true");
			redoMenuItem.setAttribute("disabled", "true");
		}
		else {
			if (WordCountPlus.undoAllowed) {
				undoMenuItem.setAttribute("disabled", "false");
				redoMenuItem.setAttribute("disabled", "true");
			}
			else {
				undoMenuItem.setAttribute("disabled", "true");
				redoMenuItem.setAttribute("disabled", "false");
			}
		}		
		
		// Reset to Zero
		var resetMenuItem = document.getElementById("wordcountplus-reset");
		if (WordCountPlus.totalWords == 0 && WordCountPlus.selectedWords == 0) {
			resetMenuItem.setAttribute("disabled", "true");
		}
		else {		
			resetMenuItem.setAttribute("disabled", "false");
		}		
		
		// Copy Words
		var copyselectedMenuItem = document.getElementById("wordcountplus-copyselected");
		if (WordCountPlus.selectedWords == 0) {
			copyselectedMenuItem.setAttribute("disabled", "true");
		}
		else {		
			copyselectedMenuItem.setAttribute("disabled", "false");
		}	
		
		// Copy Total
		var copytotalMenuItem = document.getElementById("wordcountplus-copytotal");
		if (WordCountPlus.totalWords == 0) {
			copytotalMenuItem.setAttribute("disabled", "true");
		}
		else {		
			copytotalMenuItem.setAttribute("disabled", "false");
		}	
		
		// Auto Copy
		var autocopyMenuItem = document.getElementById("wordcountplus-autocopy");
		autocopyMenuItem.setAttribute("checked", WordCountPlus.autoCopy);
		
		// Auto Deselect
		var autodeselectMenuItem = document.getElementById("wordcountplus-autodeselect");
		autodeselectMenuItem.setAttribute("checked", WordCountPlus.autoDeselect);
	},
		
	contextItemsHide: function() {
		var sel = WordCountPlus.getSelectedText();
		var noSelection = (sel == "") ? true : false;
		document.getElementById("wordcountplus-count").hidden = noSelection;
		document.getElementById("wordcountplus-add").hidden = noSelection;
	},
		
	toggleAutoCopy: function(e) {
		e.preventDefault();
		e.stopPropagation();
		var ac = WordCountPlus.autoCopy; 
		document.getElementById("wordcountplus-autocopy").checked = !ac;
		WordCountPlus.autoCopy = !ac;
		WordCountPlus.prefs.setBoolPref("extensions.wordcountplus.autoCopy", !ac);		
	},
	
	toggleAutoDeselect: function(e) {
		e.preventDefault();
		e.stopPropagation();
		var ads = WordCountPlus.autoDeselect;
		document.getElementById("wordcountplus-autodeselect").checked = !ads;
		WordCountPlus.autoDeselect = !ads;
		WordCountPlus.prefs.setBoolPref("extensions.wordcountplus.autoDeselect", !ads);		
	},
	
	getCounterState: function() {	
		WordCountPlus.selectedWords = 0;		
		WordCountPlus.totalWords = 0;
		WordCountPlus.previousTotal = -1;
		WordCountPlus.selectedChars = 0;		
		WordCountPlus.totalChars = 0;
		WordCountPlus.previousCharTotal = -1;
		WordCountPlus.undoAllowed = false;
		WordCountPlus.autoCopy = false;
		WordCountPlus.autoDeselect = false;
		
		try {	
			WordCountPlus.selectedWords = WordCountPlus.prefs.getIntPref("extensions.wordcountplus.selectedWords");
			WordCountPlus.totalWords = WordCountPlus.prefs.getIntPref("extensions.wordcountplus.totalWords");
			WordCountPlus.previousTotal = WordCountPlus.prefs.getIntPref("extensions.wordcountplus.previousTotal");
			WordCountPlus.selectedChars = WordCountPlus.prefs.getIntPref("extensions.wordcountplus.selectedChars");
			WordCountPlus.totalChars = WordCountPlus.prefs.getIntPref("extensions.wordcountplus.totalChars");
			WordCountPlus.previousCharTotal = WordCountPlus.prefs.getIntPref("extensions.wordcountplus.previousCharTotal");
			WordCountPlus.undoAllowed = WordCountPlus.prefs.getBoolPref("extensions.wordcountplus.undoAllowed");			
			WordCountPlus.autoCopy = WordCountPlus.prefs.getBoolPref("extensions.wordcountplus.autoCopy");			
			WordCountPlus.autoDeselect = WordCountPlus.prefs.getBoolPref("extensions.wordcountplus.autoDeselect");
		}
		catch(e) {}
	},
	
	getOptions: function() {
		WordCountPlus.preCountText = "";
		WordCountPlus.postCountText = "";
		WordCountPlus.thousandSep = "";
    	
		try {
			WordCountPlus.preCountText = WordCountPlus.options.getCharPref("textBeforeCount");
			WordCountPlus.postCountText = WordCountPlus.options.getCharPref("textAfterCount");
			WordCountPlus.thousandSep = WordCountPlus.options.getCharPref("thousandSep");
		}
		catch(e) {}
	},
	
	doCopy: function(numWords) {
		var numString = WordCountPlus.customFormat(numWords);
		WordCountPlus.clipboardHelper.copyString(numString);
	},
	
	customFormat: function(numWords) {
		var numString = numWords;
		if (numString != "") {
			numString = WordCountPlus.addThousandSep(numWords);
			numString = WordCountPlus.preCountText + numString + WordCountPlus.postCountText;
		}
		return numString;
	},
	
	addThousandSep: function(numWords) {
		var numString = numWords.toString();
		if ((numString != "") && (WordCountPlus.thousandSep != "")) {
			numString = numString.replace(/(\d)(?=(\d{3})+([.]|$))/g,"$1"+WordCountPlus.thousandSep);
		}
		return numString;
	},
	
	observe: function(subject, topic, data) {
		// Respond to pertinent notifications
		switch(topic) {
			case "nsPref:changed":
			// The preferences were changed in the Options dialogue
			WordCountPlus.getOptions();
			WordCountPlus.updateStatus();
			break;
			
			case "quit-application-granted":
			// Reset counts to zero and disable undo functionality
			WordCountPlus.prefs.setIntPref("extensions.wordcountplus.selectedWords", 0);
			WordCountPlus.prefs.setIntPref("extensions.wordcountplus.totalWords", 0);
			WordCountPlus.prefs.setIntPref("extensions.wordcountplus.previousTotal", -1);
			WordCountPlus.prefs.setIntPref("extensions.wordcountplus.selectedChars", 0);
			WordCountPlus.prefs.setIntPref("extensions.wordcountplus.totalChars", 0);
			WordCountPlus.prefs.setIntPref("extensions.wordcountplus.previousCharTotal", -1);
			WordCountPlus.observerService.removeObserver(wcp_quitObserver, "quit-application-granted");
			break;
			
			default:
			// do nothing
		}
	}
		
}

window.addEventListener("load", WordCountPlus.init, false);
window.addEventListener("focus", WordCountPlus.showPanel, false);